//////////////////////////////////////////////////////////////////////
// JetSQL - MS Jet SQL script interpreter (c) TECHSYS HW a SW s.r.o.
//////////////////////////////////////////////////////////////////////
// Modul:  astrings.h
// Author: Pavel Klocek
// Desc:   string manipulation functions implementation
//////////////////////////////////////////////////////////////////////

//@doc
/*@module ASTRINGS.CPP : string manipulation functions |
 * This module contains the functions for inserting string to another
 * string, for wraping words from strings and for filename parsing.
 */

#include "stdafx.h"
#include "ctype.h"

/////////////////////////////////////////////////////////////////////////////
//common string manipulations

/*@func BOOL | InsInString | 
 *Inserts lpszText in string S at position pos
 *
 * @parm CString& | S | Reference to a CString to insert into
 * @parm const char* | lpszText | Inserted text
 * @parm int | pos | Position in the string S
 * 
 * @rdesc Returns TRUE if successfull, FALSE otherwise
 */
BOOL InsInString(CString& S, const char* lpszText,int pos)
{
 int len = S.GetLength();
 if(len<pos)
 {
  TRACE0("WARNING: astrings::InsertText >> invalid inserting position\n");
  return FALSE;
 }
 S = S.Left(pos)+lpszText+S.Right(len-pos);
 return TRUE;
}

/*@func
 * Replaces tabs in S with spaces, assumes equidistant tab stops
 * by "dist" columns.                        
 *
 * @parm CString& | S | String with text to expand
 * @parm int | dist | Distance of Tabstops
 */
void ExpandTabs(CString& S, int dist)
{
 CString Ex = "";  			//expanded string
 char ch;                   //current source character
 int len = S.GetLength();

 for(int ix=0; ix<len; ix++)
 {
  ch = S[ix];
  if(ch=='\t')
  {
   Ex += " ";
   while((Ex.GetLength()%dist)) Ex += " ";
  }
  else
  {
   Ex += ch;
  }
 }
 S = Ex;
}
           
BOOL isDiv(char ch)
{
  if((ch==' ')||(ch=='\n')||(ch==';'))return TRUE;
  return FALSE;
}             

/*@func
 * Inserts line breaks to divide text longer than 80 characters.
 *
 * @parm CString& | S | String with text to divide
 * @parm int | spos | Zerobased position in line for the first character in string.
 * For spos>0, the first line will be smaller.
 */
void SplitLine(CString& S, int spos)
{                      
	ASSERT(spos<80);
  
	int maxlen = 80-spos;
	int lasteol = 0;
	int lastdiv = -1;
	int curpos = 0;

	while(curpos < S.GetLength())
	{
		if(isDiv(S[curpos])) lastdiv = curpos;
		if((curpos-lasteol)>=maxlen)
		{           
			if(!(lastdiv>lasteol))
			{
				for(;;)
				{
					curpos++;
					if(curpos==S.GetLength()) return;
					if(isDiv(S[curpos]))
					{
						lastdiv = curpos;
						break;
					}
				}
			}
			S = S.Left(lastdiv+1)+"\r\n"+S.Right(S.GetLength()-lastdiv-1);
			lasteol = lastdiv+3;
			curpos = lasteol;
			maxlen = 80;
		}
		else curpos++;
	}
}

/*@func
 * Trims given characters from begin and end of the string.
 *
 * @parm CString& | S | String with text to divide
 * @parm char | ch | Character, that will be trimmed. Implicitly set to a space character.
 */
void TrimString(CString& S, char ch /*= ' '*/)
{       
	int ix;           
	int len = S.GetLength();                          
	//trim the characters from the begin of the string
	for(ix=0;ix<len;ix++)
	{
		if(S[ix]!=ch)break;
	}	                   
	if(ix) S = S.Right(len-ix);
	//trim the characters from the end of the string  
	len = S.GetLength();
	for(ix=len-1;ix>=0;ix--)
	{
		if(S[ix]!=ch)break;
	}	                   
	S = S.Left(ix+1);
	
}

/////////////////////////////////////////////////////////////////////////////
//word wrapping

/*@func
 * Returns number of words in string. 
 *
 * @parm const char* | lpszLine | Text to count words within
 *
 * @ex Split string to words |
 * 
 * CString text = "hi boy!";
 * int wnum = GetNumWords(text);
 * for(int ix=0; ix<wnum; ix++) printf("%s\n",GetWord(text,ix);
 */
int GetNumWords(const char* lpszLine)
{
#ifdef _DEBUG
 if(lpszLine==NULL) TRACE0("astrings: GetNumWords(NULL)\n");
 ASSERT(lpszLine);
#endif

 int count;

 
 __asm
 {
   //count <=> BX
   //inSpace <=> CX

   //*** initialization ***
   push EDI
   mov EBX,0             //count = 0;
   mov CX,1             //inSpace = TRUE;
   mov EDI,lpszLine      //read string pointer into the processor
   //*** searching in string ***
 lbl1: 
   mov AL,[EDI]       //read curent char into the processor
   and AL,AL
   jz lbl4              //end of string found
   //test current char for dividier
   cmp AL,' '
   je lbl2              //current char is a dividier
   cmp AL,','
   je lbl2              //current char is a dividier
   //current char isn't a dividier
   and CX,CX
   jz lbl3              //previous char wasn't a dividier
   //begin of a word
   inc EBX               //count++
   mov CX,0             //inSpace = FALSE
   inc EDI               //next char
   jmp lbl1
 lbl2:
   //current char is a dividier
   mov CX,1             //inSpace = TRUE
 lbl3:                                  
   inc EDI               //next char
   jmp lbl1
   //*** storing result ***
 lbl4:                    
   mov count,EBX
   pop EDI
 }

 /*
 count = 0;   		//number of founded words
 BOOL inSpace = TRUE;	//true, if last character was a space
 
  
 for (unsigned int i=0; i<strlen(lpszLine); i++)
 {            
  if (lpszLine[i]!=' ')
  {
   if (inSpace || lpszLine[i]==',') 
   {
    inSpace = FALSE;
    count++;
   }
  }
  else inSpace = TRUE;
 }
*/
  return count; 
}

/* @func  CString | GetWord |
 * Extracts ix-th word from a string. Word dividers are space and comma.
 *
 * @parm const char* | lpszLine | Text to find the word within
 *
 * @parm int | ix | Zerobased index of the word
 *
 * @rdesc Returns the word or an empty string if not found.
 *
 * @ex Split string to words |
 * 
 * CString text = "hi boy!";
 * int wnum = GetNumWords(text);
 * for(int ix=0; ix<wnum; ix++) printf("%s\n",GetWord(text,ix);
 */
CString GetWord(const char* lpszLine, int ix)
{
 #ifdef _DEBUG
  if(lpszLine==NULL) TRACE0("astrings: GetWord(NULL,...)\n");
  ASSERT(lpszLine);
 #endif

 char* buf = new char[strlen(lpszLine)+1];
 ASSERT(buf);

 __asm
 {
      //DX <=> index of the searched word
      //BX <=> index of the founded word
      //CX <=> inSpace

      //*** initialize ***
      push EDI
      mov EDX,ix
      xor EBX,EBX
	  mov CX,1	//inSpace = TRUE;
      mov EDI,lpszLine        //read the string pointer into the processor
      //*** search for bogin of the word ***
  lbl1:
      mov AL,[EDI]         //read curent char into the processor
      and AL,AL
      jz nofnd               //end of string found
      //test current char for dividier
      cmp AL,' '
      je lbl2                //current char is a dividier
      cmp AL,','
      je lbl2                //current char is a dividier
      //current char isn't a dividier
      and CX,CX
      jz lbl3                //previous char wasn't a dividier
      //begin of a word
      cmp EBX,EDX
      je fnd                 //begin of the word is found
      //not searched word
      inc EBX
      mov CX,0               //inSpace = FALSE
      inc EDI
      jmp lbl1               //next char
  lbl2:
      //current char is a dividier
      mov CX,1               //inSpace = TRUE
  lbl3:
      inc EDI
      jmp lbl1               //next char
      //*** searched word wasn't found ***
  nofnd:
      push DS
      push ESI
      mov ESI,buf             //read result string pointer to the processor
      mov [ESI],0          //result is an emty string
      jmp lblend
      //*** extract searched word into buf ***
  fnd:
      push DS
      push ESI
      mov ESI,buf             //read result string pointer to the processor
  lbl4:
      mov AL,[EDI]         //read curent char into the processor
      //test current char for dividier
      cmp AL,' '
      je lbl5                //current char is a dividier
      cmp AL,','
      je lbl5                //current char is a dividier
      //current char isn't a dividier
      mov [ESI],AL         //copy current char into buf
      and AL,AL
      jz lblend               //end of string found
      inc EDI
      inc ESI
      jmp lbl4                //next char
      //current char is a dividier
  lbl5:
      mov [ESI],0           //ends string in buf
      //*** fixing results ***
  lblend:
      pop ESI
      pop DS
      pop EDI
 }

 CString line = buf;
 delete [] buf;
/*
  CString line = lpszLine;
  line += ' ';  //must return "" for ix>number of words

  for(int i=0;i<=ix;i++)
  {
    //removing spaces from begin of line
    while((!line.IsEmpty())&&(line[0]==' ')) line = line.Right(line.GetLength()-1);
    //cutting of string
    if(i<ix) line = line.Right(line.GetLength()-line.Find(' '));
    else if(line.Find(' ')>=0) line = line.Left(line.Find(' '));
  }
 */ 
  return line;
}

/* @func CString | GetWords |
 * Returns string with all words beginning from word with an index "ix"
 *
 * @parm const char* | lpszLine | Text to find the words within
 * @parm int | ix | Zerobased index of the first word
 */
CString GetWords(const char* lpszLine, int ix)
{
 #ifdef _DEBUG
  if(lpszLine==NULL) TRACE0("astrings: GetWords(NULL,...)\n");
  ASSERT(lpszLine);
 #endif

  CString line = lpszLine;
  line += ' ';  //must return "" for ix>unmber of words

  for(int i=0;i<=ix;i++)
  {
    //removing spaces from begin of line
    while((!line.IsEmpty())&&((line[0]==' ')||(line[0]==','))) line = line.Right(line.GetLength()-1);
    //cutting of string 
    int div1,div2;
    div1 = line.Find(' ');
    div2 = line.Find(',');
    if(div1<0) div1 = div2;
    else if((div2>0)&&(div2<div1)) div1 = div2;
    if(i<ix) line = line.Right(line.GetLength()-div1);
  }

  return line;
}

/////////////////////////////////////////////////////////////////////////////
//filename and path manipulating

/*func
 * Returns string with file extension.
 *
 * @parm const char* | lpszLine | Text with a path
 */
CString GetExtension(const char* lpszLine)
{
 #ifdef _DEBUG
  if(lpszLine==NULL) TRACE0("astrings: GetExtension(NULL)\n");
  ASSERT(lpszLine);
 #endif

 CString line = lpszLine;
 if(line.ReverseFind('.')>=0)
 {
  line = line.Right(line.GetLength()-line.ReverseFind('.')-1);
//  if(line.GetLength()>3) line = line.Left(3);
 }
 else line.Empty();
 return line;
}

/*@func
 * Returns string with filename (8+3)
 */
CString GetFilename(const char* lpszLine)
{
 #ifdef _DEBUG
  if(lpszLine==NULL) TRACE0("astrings: GetFilename(NULL,...)\n");
  ASSERT(lpszLine);
 #endif 

 CString line = lpszLine;  
 if(line.ReverseFind('\\')>=0)
 {
  line = line.Right(line.GetLength()-line.ReverseFind('\\')-1);
 }                                          
 else 
  if(line.ReverseFind(':')>=0)
  {
   line = line.Right(line.GetLength()-line.ReverseFind(':')-1);
  }                                          
 
 return line;
}

/*@func
 * Returns string with path without filename.
 *
 * @parm const char* | lpszLine | Text with a path
 * @parm BOOL | bEndBacksl | If TRUE, the text can end with backslash. Default value is TRUE.
*/
CString GetPath(const char* lpszLine, BOOL bEndBacksl /*=TRUE*/)
{
 #ifdef _DEBUG
  if(lpszLine==NULL) TRACE0("astrings: GetPath(NULL,...)\n");
  ASSERT(lpszLine);
 #endif

 CString line = lpszLine;
 if(line.ReverseFind('\\')>=0)
 {
  if(bEndBacksl) line = line.Left(line.ReverseFind('\\')+1);
  else line = line.Left(line.ReverseFind('\\'));
 }
 else
  if(line.ReverseFind(':')>=0)
  {
   line = line.Left(line.ReverseFind(':')+1);
  }
  else line = "";
 return line;
}

/*@func
 * Replaces the file extension.
 *                                                     
 * @parm CString& | Line | String with path to change
 * @parm const char* | lpszExt | Text with an extension
 */
void ChangeExtension(CString& Line, const char* lpszExt)
{
 #ifdef _DEBUG
  if(lpszExt==NULL) TRACE0("astrings: ChangeExtension(...,NULL)\n");
  ASSERT(lpszExt);
 #endif

 if (Line.ReverseFind('.')>=0) Line = Line.Left(Line.ReverseFind('.'));
 if(strlen(lpszExt))
 {
  Line += '.';
  Line += lpszExt;
 } 
}

//checks, if the string is valid problem name
BOOL CheckProblemName(CString fname)
{                  
 if(fname.IsEmpty())return FALSE;
 if(fname.GetLength()>8) return FALSE;                      //too long
 if (!GetExtension(fname).IsEmpty()) return FALSE;
 for(int i=0;i<fname.GetLength();i++) if((!__iscsym(fname[i]))&&(fname[i]!='!')&&(fname[i]!='$')&&(fname[i]!='~'))return FALSE;
 return TRUE;
}

