//////////////////////////////////////////////////////////////////////
// JetSQL - MS Jet SQL script interpreter (c) TECHSYS HW a SW s.r.o.
//////////////////////////////////////////////////////////////////////
// Module: main.cpp
// Author: Pavel Klocek
// Desc:   main function 
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Interpreter.h"

using namespace std;

extern char * stristr( char * pszSource, const char * pcszSearch );

//callback for displaying messages
void show(int line, const char *txt, SQLSHOWTYPE type)
{
	switch(type)
	{
	case fatalerror:
		cout <<"*** "<<txt<<endl<<endl;
		break;
	case error:
		cout <<"*** Line "<<line<<": "<<txt<<endl<<endl;
		break;
	case warning:
		cout <<"### Line "<<line<<": "<<txt<<endl<<endl;
		break;
	case info:
		cout <<txt<<endl<<endl;
		break;
	case list:
		cout <<txt<<endl;
		break;
	case echoln:
		cout <<txt<<endl;
		break;
	}
}

//returns number of errors in SQL script or -1 for fatal error
//(like missing file)
int main(int argc ,char *argv[ ])
{
	int res = 0; //program return code
	
	bool create = false;
	bool verbose = false;
	bool echo = false;
	CString sqlfile,mdbfile;

	//program title
	cout << "JetSQL v0.1 (c)1999 TECHSYS HW & SW s.r.o."<<endl<<endl;

	//command line help if no arguments are present
	if(argc==1 || stristr(argv[1],"-h") || stristr(argv[1],"/h") || stristr(argv[1],"-?") || stristr(argv[1],"/?"))
	{
		cout << "JetSQL - SQL script interpeter for MS Jet database engine" << endl;
		cout << "Version 0.1 (c)1999 TECHSYS HW & SW s.r.o." << endl << endl;
		cout << "Command line syntax:" << endl << endl;
		cout << "JetSQL [options] sqlfile[.sql] mdbfile[.mdb]"<<endl<<endl;
		cout << "  options:"<<endl;
		cout <<"    -c ... new database file is created (existing file is rewrited)"<<endl;
		cout <<"    -v ... verbose mode"<<endl;
		cout <<"    -e ... echo on (prints executed commands)"<<endl;
		cout <<"    -mMACRO=VALUE ... defines macro (occurences of %MACRO% are replaced"<<endl;
		cout <<"                        by VALUE before executing commands)"<<endl<<endl;
		cout << "  sqlfile ... SQL script file with extension .sql"<<endl<<endl;
		cout << "  mdbfile ... database file with extension .mdb"<<endl<<endl;
		cout << "Example: JetSQL -mEXTFILE=c:\\db\\data.mdb test test"<<endl;
		cout << "         JetSQL -mEXTFILE=c:\\db\\data.mdb test.sql test.mdb - is the same"<<endl;
		cout << "         JetSQL -c -mPARAM=\"two words\" test test - is the same"<<endl<<endl;

		return 0;
	}

	//Initialize the database engine
	AfxDaoInit();
	
	//script interpreter, contains an instance of CDaoDatabase class, 
	//therefore must be dynamic and destroyed before AfxDaoTerm() is called
	CSqlInterpreter* interpreter = new CSqlInterpreter();

  try{
	//parsing the command line
	int arg = 1; //start with the first argument
	//parsing options
	while(arg<=argc && (argv[arg][0]=='-' || argv[arg][0]=='/'))
	{
		//handle option
		switch(argv[arg][1])
		{
		case 'c':
		case 'C':
			create = true;
			break;
		case 'e':
		case 'E':
			echo = true;
			break;
		case 'v':
		case 'V':
			verbose = true;
			break;
		case 'm':
		case 'M':
			//extract macro
			{
				char* value = strchr(argv[arg]+2,'=');
				if(!value)
				{
					cout << "*** invalid macro option:"<<argv[arg]<<endl<<endl;
					throw 4;
				}
				*value = 0;
				value++;
				interpreter->AddMacro(argv[arg]+2,value);
			}
			break;
		default:
				cout << "*** ignore unknown option: "<<argv[arg]<<endl<<endl;
		}
		//next argument
		arg++;
	}

	//parsing filenames
	if(arg>argc) 
	{
		cout << "*** filenames missing"<<endl<<endl;
		throw 5;
	}
	sqlfile = argv[arg];
	ChangeExtension(sqlfile,"sql");
	arg++;
	if(arg>argc) 
	{
		cout << "*** database filename missing"<<endl<<endl;
		throw 6;
	}
	mdbfile = argv[arg];
	ChangeExtension(mdbfile,"mdb");

	//running the script
	res = interpreter->Go(mdbfile,sqlfile,show,create,verbose,echo);
	if(res>0) cout << res << " errors" <<endl<<endl;
	else if(res==0) cout << "No errors"<<endl<<endl;

  }
  catch(...)
  {
		res = -1;
  }
	
  //destroying objects
  delete interpreter;
  AfxDaoTerm();
	
  return res;
}




